@@ -1,58 +1,19 @@
-package org.apache.commons.net.tftp;
-
-/* ====================================================================
- * The Apache Software License, Version 1.1
+/*
+ * Copyright 2001-2005 The Apache Software Foundation
  *
- * Copyright (c) 2001 The Apache Software Foundation.  All rights
- * reserved.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Apache" and "Apache Software Foundation" and
- *    "Apache Commons" must not be used to endorse or promote products
- *    derived from this software without prior written permission. For
- *    written permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without
- *    prior written permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
+package org.apache.commons.net.tftp;
 
 import java.io.IOException;
 import java.io.InputStream;
@@ -67,17 +28,17 @@ import org.apache.commons.net.io.ToNetASCIIInputStream;
 /***
  * The TFTPClient class encapsulates all the aspects of the TFTP protocol
  * necessary to receive and send files through TFTP.  It is derived from
- * the <a href="org.apache.commons.net.tftp.TFTP.html"> TFTP class </a> because
- * it is more convenient than using aggregation, and as a result exposes 
+ * the {@link org.apache.commons.net.tftp.TFTP} because
+ * it is more convenient than using aggregation, and as a result exposes
  * the same set of methods to allow you to deal with the TFTP protocol
  * directly.  However, almost every user should only be concerend with the
- * the <a href="org.apache.commons.net.DatagramSocketClient.html#open"> open() </a>,
- * <a href="org.apache.commons.net.DatagramSocketClient.html#close"> close() </a>,
- * <a href="#sendFile"> sendFile() </a>, and
- * <a href="#receiveFile"> receiveFile() </a> methods.  Additionally, the
- * <a href="#setMaxTimeouts"> setMaxTimeouts() </a> and
- * <a href="org.apache.commons.net.DatagramSocketClient.html#setDefaultTimeout">
- * setDefaultTimeout() </a> methods may be of importance for performance
+ * the {@link org.apache.commons.net.DatagramSocketClient#open  open() },
+ * {@link org.apache.commons.net.DatagramSocketClient#close  close() },
+ * {@link #sendFile  sendFile() }, and
+ * {@link #receiveFile  receiveFile() } methods.  Additionally, the
+ * {@link #setMaxTimeouts  setMaxTimeouts() } and
+ * {@link org.apache.commons.net.DatagramSocketClient#setDefaultTimeout setDefaultTimeout() }
+ *  methods may be of importance for performance
  * tuning.
  * <p>
  * Details regarding the TFTP protocol and the format of TFTP packets can
@@ -125,7 +86,7 @@ public class TFTPClient extends TFTP
      ***/
     public void setMaxTimeouts(int numTimeouts)
     {
-        if (__maxTimeouts < 1)
+        if (numTimeouts < 1)
             __maxTimeouts = 1;
         else
             __maxTimeouts = numTimeouts;
@@ -147,7 +108,7 @@ public class TFTPClient extends TFTP
      * Requests a named file from a remote host, writes the
      * file to an OutputStream, closes the connection, and returns the number
      * of bytes read.  A local UDP socket must first be created by
-     * <a href="org.apache.commons.net.DatagramSocketClient.html#open">open()</a> before
+     * {@link org.apache.commons.net.DatagramSocketClient#open open()} before
      * invoking this method.  This method will not close the OutputStream
      * containing the file; you must close it after the method invocation.
      * <p>
@@ -221,7 +182,7 @@ _receivePacket:
                 }
 
                 // The first time we receive we get the port number and
-		// answering host address (for hosts with multiple IPs)
+        // answering host address (for hosts with multiple IPs)
                 if (lastBlock == 0)
                 {
                     hostPort = received.getPort();
@@ -321,7 +282,7 @@ _receivePacket:
      * Requests a named file from a remote host, writes the
      * file to an OutputStream, closes the connection, and returns the number
      * of bytes read.  A local UDP socket must first be created by
-     * <a href="org.apache.commons.net.DatagramSocketClient.html#open">open()</a> before
+     * {@link org.apache.commons.net.DatagramSocketClient#open open()} before
      * invoking this method.  This method will not close the OutputStream
      * containing the file; you must close it after the method invocation.
      * <p>
@@ -345,7 +306,7 @@ _receivePacket:
 
     /***
      * Same as calling receiveFile(filename, mode, output, host, TFTP.DEFAULT_PORT).
-     * 
+     *
      * @param filename The name of the file to receive.
      * @param mode     The TFTP mode of the transfer (one of the MODE constants).
      * @param output   The OutputStream to which the file should be written.
@@ -362,7 +323,7 @@ _receivePacket:
 
     /***
      * Same as calling receiveFile(filename, mode, output, hostname, TFTP.DEFAULT_PORT).
-     * 
+     *
      * @param filename The name of the file to receive.
      * @param mode     The TFTP mode of the transfer (one of the MODE constants).
      * @param output   The OutputStream to which the file should be written.
@@ -384,14 +345,13 @@ _receivePacket:
      * Requests to send a file to a remote host, reads the file from an
      * InputStream, sends the file to the remote host, and closes the
      * connection.  A local UDP socket must first be created by
-     * <a href="org.apache.commons.net.DatagramSocketClient.html#open">open()</a> before
+     * {@link org.apache.commons.net.DatagramSocketClient#open open()} before
      * invoking this method.  This method will not close the InputStream
      * containing the file; you must close it after the method invocation.
      * <p>
      * @param filename The name the remote server should use when creating
      *        the file on its file system.
      * @param mode     The TFTP mode of the transfer (one of the MODE constants).
-     * @param output   The InputStream containing the file.
      * @param host     The remote host receiving the file.
      * @param port     The port number of the remote TFTP server.
      * @exception IOException If an I/O error occurs.  The nature of the
@@ -412,6 +372,7 @@ _receivePacket:
 
         dataLength = lastBlock = hostPort = bytesRead = 0;
         block = 0;
+        boolean lastAckWait = false;
 
         if (mode == TFTP.ASCII_MODE)
             input = new ToNetASCIIInputStream(input);
@@ -461,7 +422,7 @@ _receivePacket:
                 }
 
                 // The first time we receive we get the port number and
-		// answering host address (for hosts with multiple IPs)
+        // answering host address (for hosts with multiple IPs)
                 if (lastBlock == 0)
                 {
                     hostPort = received.getPort();
@@ -495,7 +456,10 @@ _receivePacket:
                         if (lastBlock == block)
                         {
                             ++block;
-                            break _receivePacket;
+                            if (lastAckWait)
+                              break _sendPacket;
+                            else
+                              break _receivePacket;
                         }
                         else
                         {
@@ -541,9 +505,8 @@ _receivePacket:
             data.setData(_sendBuffer, 4, offset - 4);
             sent = data;
         }
-        while (dataLength == 0);
+        while (dataLength == 0 || lastAckWait);
 
-        bufferedSend(sent);
         endBufferedOps();
     }
 
@@ -552,14 +515,13 @@ _receivePacket:
      * Requests to send a file to a remote host, reads the file from an
      * InputStream, sends the file to the remote host, and closes the
      * connection.  A local UDP socket must first be created by
-     * <a href="org.apache.commons.net.DatagramSocketClient.html#open">open()</a> before
+     * {@link org.apache.commons.net.DatagramSocketClient#open open()} before
      * invoking this method.  This method will not close the InputStream
      * containing the file; you must close it after the method invocation.
      * <p>
      * @param filename The name the remote server should use when creating
      *        the file on its file system.
      * @param mode     The TFTP mode of the transfer (one of the MODE constants).
-     * @param output   The InputStream containing the file.
      * @param hostname The name of the remote host receiving the file.
      * @param port     The port number of the remote TFTP server.
      * @exception IOException If an I/O error occurs.  The nature of the
@@ -576,13 +538,11 @@ _receivePacket:
 
     /***
      * Same as calling sendFile(filename, mode, input, host, TFTP.DEFAULT_PORT).
-     * 
+     *
      * @param filename The name the remote server should use when creating
      *        the file on its file system.
      * @param mode     The TFTP mode of the transfer (one of the MODE constants).
-     * @param output   The InputStream containing the file.
-     * @param hostname The name of the remote host receiving the file.
-     * @param port     The port number of the remote TFTP server.
+     * @param host     The name of the remote host receiving the file.
      * @exception IOException If an I/O error occurs.  The nature of the
      *            error will be reported in the message.
      * @exception UnknownHostException  If the hostname cannot be resolved.
@@ -596,13 +556,11 @@ _receivePacket:
 
     /***
      * Same as calling sendFile(filename, mode, input, hostname, TFTP.DEFAULT_PORT).
-     * 
+     *
      * @param filename The name the remote server should use when creating
      *        the file on its file system.
      * @param mode     The TFTP mode of the transfer (one of the MODE constants).
-     * @param output   The InputStream containing the file.
      * @param hostname The name of the remote host receiving the file.
-     * @param port     The port number of the remote TFTP server.
      * @exception IOException If an I/O error occurs.  The nature of the
      *            error will be reported in the message.
      * @exception UnknownHostException  If the hostname cannot be resolved.
